@@ -79,35 +79,48 @@ local function in_scope(var, index)
 end
 
 local function contains_call(node)
+   if node._contains_call ~= nil then
+      -- return cached result
+      return node._contains_call
+   end
+
    if node.tag == "Call" or node.tag == "Invoke" then
+      node._contains_call = true
       return true
    end
 
    if node.tag ~= "Function" then
       for _, sub_node in ipairs(node) do
          if type(sub_node) == 'table' and contains_call(sub_node) then
+            node._contains_call = true
             return true
          end
       end
    end
 
+   node._contains_call = false
    return false
 end
 
 local function is_circular_reference(item, var)
+   -- OpSet is circular by definition
+   if not (item.tag == "Set" or item.tag == "Local") then
+      return false
+   end
+
    -- No support for matching multiple assignment to the specific assignment
-   if not (item.tag == "Set" or item.tag == "Local" or item.tag == "OpSet") then
+   if not item.lhs or #item.lhs ~= 1 or not item.rhs or #item.rhs ~= 1 then
       return false
    end
 
    -- Case t[t.function()] = t.func()
    -- Functions can have side-effects, so this isn't purely circular
-   local right_assignment = item.tag == "OpSet" and item.rhs or item.rhs[1]
+   local right_assignment = item.rhs[1]
    if contains_call(right_assignment) then
       return false
    end
 
-   local left_assignment = item.tag == "OpSet" and item.lhs or item.lhs[1]
+   local left_assignment = item.lhs[1]
    if contains_call(left_assignment) then
       return false
    end
